import json
import unittest
from jsonfinder import *


class BasicTests(unittest.TestCase):

    def setUp(self):
        with open('tests/testin.txt') as infile:
            self.string = infile.read()

    def test_finder(self):
        result = []
        for start, end, obj in jsonfinder(self.string):
            if obj is not None:
                result.append(json.dumps(obj, indent=2, sort_keys=True))
            else:
                result.append(self.string[start:end])

        with open('tests/testout.txt') as outfile:
            self.assertEquals("\n".join(result), outfile.read())

    def test_has_json(self):
        self.assertTrue(has_json(self.string))
        self.assertTrue(has_json("hi { [1] stuff"))
        self.assertFalse(has_json("hi { [1,] stuff"))
        self.assertFalse(has_json("a normal string"))
        self.assertFalse(has_json(""))

    def test_only_json(self):
        self.assertRaises(ValueError, lambda: only_json(self.string))
        self.assertEquals(only_json('prefix {"a":"b"} suffix'), (7, 16, {'a': 'b'}))
        self.assertEquals(only_json('true }{ {{ true }} [1,2,3] false 1 2 3 null'), (19, 26, [1, 2, 3]))

    def test_type_error(self):
        # noinspection PyTypeChecker
        self.assertRaises(AttributeError, lambda: has_json(123))

    def test_start_end_match(self):
        prev = 0
        end = None
        for start, end, obj in jsonfinder(self.string):
            self.assertEquals(start, prev)
            prev = end
        self.assertEquals(end, len(self.string))

    def test_empty_object(self):
        self.assertFalse(any(t[2] for t in jsonfinder("{}")))
        self.assertEquals(len(list(jsonfinder("{}"))), 3)

    def test_min_elemets(self):
        obj = {"a": [1, 2], "b": [3, [4, {"c": 5, "d": [6, 7, [8], {"e": 9}]}, 10]]}
        size = 10
        for test_size in xrange(size*2):
            self.assertEquals(check_min_elements(obj, test_size), test_size <= size)


suite = unittest.TestSuite()
suite.addTest(unittest.makeSuite(BasicTests))
import jsonfinder as package
import doctest
suite.addTest(doctest.DocTestSuite(package))
unittest.TextTestRunner(verbosity=1).run(suite)


#! /usr/bin/env python

from itertools import imap
import sys
from json import dumps
from __init__ import jsonfinder, check_min_elements
from optparse import OptionParser


def parse_args(args=None, raise_option_exceptions=False):

    parser = OptionParser(usage="Usage: [python -m] jsonfinder [OPTIONS] [FILTERS]",
                          description="Detect JSON in the input and format it or filter based on its presence. "
                                      "Optionally specify space-separated filters so that the program will only pay "
                                      "attention to JSON where the source contains all the filters as substrings.")

    parser.add_option("-i", "--infile", help="Read data from INFILE instead of stdin")

    parser.add_option("-o", "--outfile", help="Write output to OUTFILE instead of stdout")

    parser.add_option("-d", "--delete",
                      help="Delete the given sections in the output. The output is partitioned into three sections:\n"
                           "J/json: JSON that has been detected and selected by the given FILTERS values.\n"
                           "C/context: data/text not within the json section but in the same line(s).\n"
                           "L/other-lines: lines not containing any JSON.\n"
                           "The values can be specified using either the capital letters or long names separated by "
                           "commas. For example, if you want the output to contain json only, you can specify:\n"
                           "-dCL  or  --delete=context,other-lines.")

    delete_options_short_to_long = {
        "J": "json",
        "C": "context",
        "L": "other-lines"
    }

    parser.add_option("-f", "--format", type="choice", choices="on off mini tiny".split(), default="on",
                      help="Choose the output format of the JSON. The possible values are:\n"
                           "on (default): pretty-print values on separate lines with indentation (see -n/--indent).\n"
                           "off: output as in input.\n"
                           "mini: shrink JSON into a single line of output, while keeping spaces after the , and : "
                           "separators.\n"
                           "tiny: like mini, but without any extra spaces at all.\n"
                           "Unless FORMAT is off, the keys will be sorted lexicographically in the output.")

    parser.add_option("-n", "--indent", type="int", default=4,
                      help="Number of spaces in each level of indentation when FORMAT is on. Default is 4.")

    parser.add_option("-l", "--linewise", action="store_true",
                      help="By default the program reads in all input at once and parses the result in memory. If the "
                           "input is large and you need a live stream of results then set this flag. However JSON "
                           "that is split across multiple lines will not be detected, meaning that FORMAT=mini/tiny "
                           "will probably not do what you want.")

    parser.add_option("-m", "--min-size", type="int", default=2, metavar="MIN",
                      help="Only pay attention to (in a similar manner to the filters) objects/arrays with at least "
                           "MIN elements. This prevents things like [1] from being recognised, which you probably "
                           "don't want. Default is 2.")

    parser.add_option("-a", "--array", action="store_true",
                      help="Delete context and other lines and output a JSON array of the selected "
                           "JSON objects/arrays. Currently the -l/--linewise flag has no effect when this is set.")

    options, args = parser.parse_args(args)

    try:
        delete = options.delete
        if delete:
            assert isinstance(delete, str)
            if len(delete) <= 3:
                try:
                    delete = {delete_options_short_to_long[opt] for opt in delete}
                except KeyError:
                    raise OptionException("The only short options allowed for DELETE are J, C, and L.")
            else:
                delete = set(delete.split(","))
                if not delete.issubset(delete_options_short_to_long.values()):
                    raise OptionException("The only long options allowed for DELETE are json, context, "
                                          "and other-lines, and they must be separated by only a comma.")
        else:
            delete = ()
        options.delete = delete

    except OptionException as exc:
        if raise_option_exceptions:
            raise
        else:
            parser.error(exc.message)

    return options, args


class OptionException(Exception):
    pass


def process_files(infile, outfile, options, filters):
    incl_other = "other-lines" not in options.delete
    incl_context = "context" not in options.delete
    incl_json = "json" not in options.delete

    if options.format != "off":
        format_json = lambda json: dumps(json, sort_keys=True,
                                         **{"on": dict(separators=(",", ": "), indent=options.indent),
                                            "mini": dict(),
                                            "tiny": dict(separators=(",", ":"))}[options.format])

    def filtered_jsonfinder(string, json_only=False):
        predicate = lambda start, end, json: (all(imap(string[start:end].__contains__, filters)) and
                                              check_min_elements(json, options.min_size))
        return jsonfinder(string, json_only=json_only, predicate=predicate)

    def process_string(string):
        for start, end, json in filtered_jsonfinder(string):
            section = string[start:end]
            if json is None:
                if options.linewise:
                    section *= incl_other if len(section) == len(string) else incl_context
                else:
                    other_start = section.find("\n")
                    if other_start == -1:
                        section *= incl_context
                    else:
                        deleted_other = "\n"
                        if start == 0:
                            deleted_other = ""
                            other_start = 0
                        if end == len(string):
                            other_end = end
                            deleted_other = ""
                        else:
                            other_end = (section.rfind("\n") + 1)
                        section = (incl_context * section[:other_start] +
                                   (section[other_start:other_end] if incl_other else deleted_other) +
                                   incl_context * section[other_end:])
            else:
                if not incl_json:
                    section = ""
                elif options.format != "off":
                    section = format_json(json)

            yield section

    def make_array_linewise():
        """
        This was an attempt to allow combining the array and linewise options, but it was proving too difficult and
        was given up on.
        """
        outfile.write("[" + (options.format == "on") * "\n")
        found_json = False
        for line in infile:
            for start, end, json in filtered_jsonfinder(line, json_only=True):
                found_json = True
                if options.format == "off":
                    json = line[start:end]
                else:
                    json = format_json(json)
                if options.format == "on":
                    json = "\n".join([options.indent * " " + json_line for json_line in json.splitlines()])
                outfile.write(found_json * "," + options.format != "tiny" * " " + json)
                if options.format == "on":
                    outfile.write("\n")
            if options.format == "off" and found_json:
                outfile.write("\n")

    def format_json_linewise():
        for line in infile:
            newline = line and line[-1] == "\n"
            if newline:
                line = line[:-1]
            result = "".join(process_string(line))
            outfile.write(result)
            if newline and result:
                outfile.write("\n")

    def format_json_all():
        part = None
        for part in process_string(infile_contents):
            outfile.write(part)
        if part and part[-1] != "\n" and infile_contents[-1] == "\n":
            outfile.write("\n")

    def make_array_all():
        array = []
        for start, end, json in filtered_jsonfinder(infile_contents, json_only=True):
            if options.format == "off":
                array.append(infile_contents[start:end])
            else:
                array.append(json)

        if options.format == "off":
            outfile.write("[" + ", ".join(array) + "]")
        else:
            outfile.write(format_json(array))
        outfile.write("\n")

    if options.linewise:
        if options.array and False:  # this combination is not currently supported
            make_array_linewise()
        else:
            format_json_linewise()
    else:
        infile_contents = infile.read()
        if options.array:
            make_array_all()
        else:
            format_json_all()

    outfile.flush()


def process_args(options, filters):

    def get_file(opt, mode, default):
        if opt:
            try:
                return open(opt, mode)
            except IOError as err:
                print >> sys.stderr, "IOError:", err
                sys.exit(1)
        else:
            return default

    infile = get_file(options.infile, "r", sys.stdin)
    outfile = get_file(options.outfile, "w", sys.stdout)

    try:
        process_files(infile, outfile, options, filters)
    finally:
        if options.infile:
            infile.close()
        if options.outfile:
            outfile.close()


def main():
    # Ignore SIG_PIPE and don't throw exceptions on it...
    # (http://docs.python.org/library/signal.html)
    from signal import signal, SIGPIPE, SIG_DFL
    try:
        signal(SIGPIPE, SIG_DFL)
    except ValueError:
        pass

    process_args(*parse_args())


if __name__ == "__main__":
    main()


from setuptools import setup

description = """\
Easily handle JSON contained within strings.
Includes a command-line tool to format JSON within the input, similar to the builtin json.tool."""


def readme():
    with open('README.rst') as readme_file:
        return readme_file.read()


setup(name='jsonfinder',
      version='0.4.1',
      description=description,
      long_description=readme(),
      classifiers=[
          "Programming Language :: Python :: 2 :: Only",
          "Topic :: Utilities"
      ],
      url="https://github.com/alexmojaki/jsonfinder",
      author='Alex Hall',
      author_email='alex.mojaki@gmail.com',
      license='MIT',
      packages=['jsonfinder'],
      include_package_data=True,
      entry_points={'console_scripts': ['jsonfinder=jsonfinder.__main__:main']})


from json import JSONDecoder

__decoder = JSONDecoder()


def jsonfinder(s, decoder=None, json_only=False, predicate=lambda s, e, o: True):
    """
    Return a generator that yields positions of JSON objects within the string s and the parsed results.

    Specifically, every iteration yields a ``(start, end, obj)`` tuple where
    ``s[start:end]`` is either a string containing no JSON objects/arrays (in which case ``obj``
    is ``None``) or consists entirely of a JSON object/array which is parsed into a python object
    contained in ``obj``. For example::

        >>> s = 'true [1,2,3] null'
        >>> for start, end, obj in jsonfinder(s):
        ...     print start, ':', end
        ...     if obj is None:
        ...         print 'String:', repr(s[start:end])
        ...     else:
        ...         print 'List of length', len(obj)
        0 : 5
        String: 'true '
        5 : 12
        List of length 3
        12 : 17
        String: ' null'
        >>> assert len(s) == end

    Note that JSON primitives are ignored if not contained within an object or array.

    Specifying ``json_only=True`` will cause plain strings to be left out of the iteration,
    i.e. ``obj`` will always be a parsed JSON object.

    Otherwise the first and last iterations will always represent strings (i.e. ``obj is None``),
    even if those strings are empty.

    Note that when testing if the current iteration is a string you probably want to write::

        if obj is not None:

    rather than::

        if obj:

    since the latter will pass if obj is an empty dict or list.

    Set ``predicate`` to a function returning True when you want a JSON object to actually be considered JSON and false
    if you want it to be merged with the strings: this is much easier than doing so manually. For example::

        >>> s = 'a list [1, 2] and an object {"a": 3} are here'
        >>> start, end, obj = next(jsonfinder(s))
        >>> s[start:end]
        'a list '
        >>> start, end, obj = next(jsonfinder(s, predicate=lambda s, e, o: isinstance(o, dict)))
        >>> s[start:end]
        'a list [1, 2] and an object '

    The arguments to the predicate are still ``start, end, obj``, just as they would be yielded, except that ``obj``
    is never None in this case.

    A customer JSONDecoder object (see the built in json module) can be passed in with the keyword
    argument ``decoder``.

    :type s: str | unicode
    :type decoder: JSONDecoder
    :rtype: __generator[(int, int, None | dict | list)]
    """

    decoder = decoder or __decoder
    string_start = find_start = 0
    while 1:
        start1 = s.find('{', find_start)
        start2 = s.find('[', find_start)
        if start1 == -1:
            if start2 == -1:
                if not json_only:
                    yield string_start, len(s), None
                return
            else:
                json_start = start2
        else:
            if start2 == -1:
                json_start = start1
            else:
                json_start = min(start1, start2)

        is_json = True
        obj = json_tuple = None

        try:
            obj, end = decoder.raw_decode(s, idx=json_start)
        except ValueError:
            is_json = False

        if is_json:
            json_tuple = (json_start, end, obj)
            if not predicate(*json_tuple):
                is_json = False

        if is_json:
            if not json_only:
                yield string_start, json_start, None
            yield json_tuple
            string_start = find_start = end
        else:
            find_start = json_start + 1


def has_json(s, decoder=None):
    """
    Return whether the given string s contains at least one JSON object/array, e.g.::

        >>> has_json('stuff {"key": "value"} things')
        True
        >>> has_json('stuff only')
        False

    A customer JSONDecoder object (see the built in json module) can be passed in with the keyword
    argument decoder.

    :type s: str | unicode
    :type decoder: JSONDecoder
    :rtype: bool
    """

    for _ in jsonfinder(s, decoder=decoder, json_only=True):
        return True
    return False


def only_json(s, decoder=None):
    """
    Return a single ``(start, end, obj)`` tuple corresponding to a parsed JSON object/array
    if the string s contains exactly one, else raise an exception, e.g.::

        >>> only_json('stuff {"key": "value"} things')[2]
        {u'key': u'value'}
        >>> only_json('stuff only')
        Traceback (most recent call last):
        ...
        ValueError: No JSON object found in argument.
        >>> only_json('{}{}')
        Traceback (most recent call last):
        ...
        ValueError: More than one JSON object found in the argument.

    A customer JSONDecoder object (see the built in json module) can be passed in with the keyword
    argument decoder.

    :type s: str | unicode
    :type decoder: JSONDecoder
    :rtype: dict | list
    """

    result = None
    for current in jsonfinder(s, decoder=decoder, json_only=True):
        if result is not None:
            raise ValueError("More than one JSON object found in the argument.")
        result = current
    if result is None:
        raise ValueError("No JSON object found in argument.")
    return result


def check_min_elements(obj, num):
    """
    Returns a boolean indicating if ``obj`` has at least ``num`` elements at its leaves when viewed as a tree.
    That is, it iterates recursively over lists and dicts, checking that the ``obj`` has at least ``num`` primitive
    values at the lowest levels. dict keys are not counted. For example::

        >>> obj = {"a": 1, "b": [2, 3]}
        >>> check_min_elements(obj, 4)
        False
        >>> check_min_elements(obj, 3)
        True

    :type num: int
    :return: bool
    """
    return __check_min_elements_helper(obj, num, 0) >= num


def __check_min_elements_helper(obj, num, count):
    values = ()
    if isinstance(obj, list):
        values = obj
    elif isinstance(obj, dict):
        values = obj.itervalues()
    else:
        count += 1

    for val in values:
        count = __check_min_elements_helper(val, num, count)
        if count >= num:
            break

    return count